@@ -1,7 +1,7 @@
 /*
  * cddb-slave.c: Implementation for the GNOME/Media/CDDBSlave2 interface.
  *
- * Copyright (C) 2001 Ximian, Inc.
+ * Copyright (C) 2001-2002 Iain Holmes.
  * 
  * Authors: Iain Holmes  <iain@ximian.com>
  */
@@ -23,6 +23,8 @@
 #include <bonobo/bonobo-exception.h>
 #include <gnome.h>
 
+#include <gconf/gconf-client.h>
+
 /* Use local copy of gnet */
 #include "gnet.h"
 
@@ -32,6 +34,8 @@
 #include "GNOME_Media_CDDBSlave2.h"
 
 
+static GConfClient *client = NULL;
+
 #define PARENT_TYPE BONOBO_OBJECT_TYPE
 static BonoboObjectClass *parent_class = NULL;
 
@@ -42,6 +46,8 @@ typedef enum {
 } CDDBSlaveAccess;
 
 struct _CDDBSlavePrivate {
+	char *name;
+	char *hostname;
 	char *server; /* Server address */
 	int port; /* Server port */
 
@@ -75,6 +81,7 @@ typedef struct _ConnectionData {
 
 	GTcpSocket *socket;
 	GIOChannel *iochannel;
+	guint tag;
 	
 	ConnectionMode mode;
 	
@@ -90,6 +97,7 @@ typedef struct _ConnectionData {
 } ConnectionData;
 
 static GHashTable *pending_requests = NULL;
+static GHashTable *cddb_cache = NULL;
 
 static void do_hello (ConnectionData *cd);
 
@@ -140,6 +148,10 @@ do_goodbye_response (ConnectionData *cd,
 	}
 
 	/* Disconnect */
+	if (cd->tag) {
+		g_source_remove (cd->tag);
+	}
+	
   	gnet_tcp_socket_unref (cd->socket);
   	g_io_channel_unref (cd->iochannel);
 	
@@ -167,6 +179,12 @@ do_goodbye_response (ConnectionData *cd,
 	return FALSE;
 }
 
+static void
+remove_entry_from_cache (const char *discid)
+{
+	g_hash_table_remove (cddb_cache, discid);
+}
+
 static void
 do_goodbye (ConnectionData *cd)
 {
@@ -187,6 +205,7 @@ do_read_response (ConnectionData *cd,
 	CDDBSlave *cddb = cd->cddb;
 	int code;
 	gboolean more = FALSE;
+	gboolean disconnect = FALSE;
 	static gboolean waiting_for_terminator = FALSE;
 	static FILE *handle = NULL;
 	
@@ -224,12 +243,24 @@ do_read_response (ConnectionData *cd,
 		fputs (response, handle);
 
 		if (response[0] == '.') {
+			CDDBEntry *entry;
+			char *filename, *dirname;
+			
 			/* Found terminator */
 			fclose (handle);
 
+			dirname = gnome_util_prepend_user_home (".cddbslave");
+			filename = g_concat_dir_and_file (dirname, cd->discid);
+			g_free (dirname);
+			
+			entry = g_hash_table_lookup (cddb_cache, cd->discid);
+			cddb_entry_parse_file (entry, filename);
+			g_free (filename);
+			
 			/* Reset the static variables for next time */
 			handle = NULL;
 			more = FALSE;
+			disconnect = TRUE;
 			waiting_for_terminator = FALSE;
 		} else {
 			more = TRUE;
@@ -241,18 +272,24 @@ do_read_response (ConnectionData *cd,
 		g_print ("Specified CDDB entry not found\n");
 		g_print ("%s\n", response);
 		more = FALSE;
+		disconnect = TRUE;
+		remove_entry_from_cache (cd->discid);
 		break;
 
 	case 402:
 		g_print ("Server error\n");
 		g_print ("%s\n", response);
 		more = FALSE;
+		disconnect = TRUE;
+		remove_entry_from_cache (cd->discid);
 		break;
 
 	case 403:
 		g_print ("Database entry is corrupt\n");
 		g_print ("%s\n", response);
 		more = FALSE;
+		disconnect = TRUE;
+		remove_entry_from_cache (cd->discid);
 		break;
 
 	case 409:
@@ -266,7 +303,7 @@ do_read_response (ConnectionData *cd,
 
 	}
 
-	if (more == FALSE) {
+	if (disconnect == TRUE) {
 		do_goodbye (cd);
 	}
 		
@@ -456,6 +493,7 @@ do_query_response (ConnectionData *cd,
 				} else {
 				/* Need to disconnect here...
 				   none of our matches matched */
+					remove_entry_from_cache (cd->discid);
 					do_goodbye (cd);
 				}
 			}
@@ -491,12 +529,14 @@ do_query_response (ConnectionData *cd,
 		g_print ("No match found\n");
 		g_print ("%s\n", response);
 		more = FALSE;
+		disconnect = TRUE;
 		break;
 
 	case 403:
 		g_print ("Database entry is corrupt\n");
 		g_print ("%s\n", response);
 		more = FALSE;
+		disconnect = TRUE;
 		break;
 
 	case 409:
@@ -510,6 +550,7 @@ do_query_response (ConnectionData *cd,
 	default:
 		g_print ("Unknown response\n");
 		g_print ("%s\n", response);
+		disconnect = TRUE;
 		more = FALSE;
 		break;
 	}
@@ -521,6 +562,11 @@ do_query_response (ConnectionData *cd,
 		do_read (cd, cat, discid, dtitle);
 	}
 
+	if (disconnect == TRUE) {
+		remove_entry_from_cache (cd->discid);
+		do_goodbye (cd);
+	}
+	
 	g_free (cat);
 	g_free (discid);
 	g_free (dtitle);
@@ -564,6 +610,8 @@ do_hello_response (ConnectionData *cd,
 		g_print ("%s\n", response);
 
 		/* Disconnect here */
+		remove_entry_from_cache (cd->discid);
+		do_goodbye (cd);
 		break;
 
 	case 402:
@@ -594,8 +642,11 @@ do_hello (ConnectionData *cd)
 	/* Send the Hello command
 	   CDDB howto says these shouldn't be hardcoded,
 	   but that seems to be a privacy issue */
-	hello = g_strconcat ("cddb hello johnsmith 198.172.174.22 ",
-			     cd->name, " ", cd->version, "\n", NULL);
+	hello = g_strdup_printf ("cddb hello %s %s %s %s\n",
+				 cd->cddb->priv->name,
+				 cd->cddb->priv->hostname,
+				 cd->name,
+				 cd->version);
 	
 	/* Need to check the return of this one */
 	status = gnet_io_channel_writen (cd->iochannel, hello,
@@ -658,6 +709,7 @@ do_open_response (ConnectionData *cd,
 	} else {
 		/* Do something to indicate that we can't contact server */
 		/* Close connection to tell listeners we're not doing anything */
+		remove_entry_from_cache (cd->discid);
 		do_goodbye (cd);
 	}
 
@@ -742,6 +794,7 @@ read_from_server (GIOChannel *iochannel,
 	return TRUE;
 
  error:
+	remove_entry_from_cache (cd->discid);
 	return FALSE;
 }
 
@@ -761,14 +814,15 @@ open_cb (GTcpSocket *sock,
 			   cd->cddb->priv->port);
 
 		/* notify listeners */
+		remove_entry_from_cache (cd->discid);
 		return;
 	}
 
 	sin = gnet_tcp_socket_get_iochannel (sock);
 	cd->iochannel = sin;
 	
-	g_io_add_watch (sin, G_IO_IN | G_IO_ERR | G_IO_HUP | G_IO_NVAL,
-			read_from_server, data);
+	cd->tag = g_io_add_watch (sin, G_IO_IN | G_IO_ERR | G_IO_HUP | G_IO_NVAL,
+				  read_from_server, data);
 }
 
 static void
@@ -791,25 +845,10 @@ cddb_send_cmd (ConnectionData *data)
 static gboolean
 cddb_check_cache (const char *discid)
 {
-	char *dirname, *discname;
-
-	g_return_val_if_fail (discid != NULL, FALSE);
-
-	dirname = gnome_util_prepend_user_home (".cddbslave");
-	if (g_file_test (dirname, G_FILE_TEST_IS_DIR) == FALSE) {
-		/* Cache dir doesn't exist */
-		mkdir (dirname, S_IRWXU);
+	if (g_hash_table_lookup (cddb_cache, discid) == NULL) {
 		return FALSE;
-	}
-
-	discname = g_concat_dir_and_file (dirname, discid);
-	g_free (dirname);
-	if (g_file_test (discname, G_FILE_TEST_IS_REGULAR) == TRUE) {
-		g_free (discname);
-		return TRUE;
 	} else {
-		g_free (discname);
-		return FALSE;
+		return TRUE;
 	}
 }
 
@@ -832,23 +871,27 @@ impl_GNOME_Media_CDDBSlave2_query (PortableServer_Servant servant,
 				   CORBA_Environment *ev)
 {
 	CDDBSlave *cddb;
+	CDDBEntry *entry;
 	ConnectionData *cd;
 	char *request, *safe_offsets;
 	char *username, *hostname, *fullname, *uri;
 
 	cddb = cddb_slave_from_servant (servant);
 
-	g_warning ("Request: %s", discid);
-	/* Do stuff */
 	if (cddb_check_cache (discid) == TRUE) {
-		g_print ("In cache\n");
 		cddb_slave_notify_listeners (cddb, discid, GNOME_Media_CDDBSlave2_OK);
 		return;
 	}
 
+	/* Make an entry */
+	entry = cddb_entry_new (discid, ntrks, offsets, nsecs);
+	g_hash_table_insert (cddb_cache, g_strdup (discid), entry);
+	
 	cd = g_new (ConnectionData, 1);
 	cd->cddb = cddb;
 	g_object_ref (cddb);
+
+	cd->tag = 0;
 	
 	cd->name = g_strdup_printf ("%s(CDDBSlave2)", name);
 	cd->version = g_strdup (version);
@@ -862,6 +905,144 @@ impl_GNOME_Media_CDDBSlave2_query (PortableServer_Servant servant,
 	cddb_send_cmd (cd);
 }
 
+static CORBA_char *
+impl_GNOME_Media_CDDBSlave2_getArtist (PortableServer_Servant servant,
+				       const CORBA_char *discid,
+				       CORBA_Environment *ev)
+{
+	CDDBEntry *entry;
+	char *split, *artist;
+	GString *dtitle;
+	CORBA_char *ret;
+	
+	entry = g_hash_table_lookup (cddb_cache, discid);
+	if (entry == NULL) {
+		/* Should set an exception here */
+		return NULL;
+	}
+
+	dtitle = g_hash_table_lookup (entry->fields, "DTITLE");
+	if (dtitle == NULL) {
+		return NULL;
+	}
+
+	split = strstr (dtitle->str, " / ");
+	if (split == NULL) {
+		ret = CORBA_string_dup (dtitle->str);
+	} else {
+		artist = g_strndup (dtitle->str, split - dtitle->str);
+		ret = CORBA_string_dup (artist);
+		g_free (artist);
+	}
+	
+	return ret;
+}
+
+static CORBA_char *
+impl_GNOME_Media_CDDBSlave2_getDiscTitle (PortableServer_Servant servant,
+					  const CORBA_char *discid,
+					  CORBA_Environment *ev)
+{
+	CDDBEntry *entry;
+	char *split;
+	GString *dtitle;
+
+	entry = g_hash_table_lookup (cddb_cache, discid);
+	if (entry == NULL) {
+		return NULL;
+	}
+
+	dtitle = g_hash_table_lookup (entry->fields, "DTITLE");
+	if (dtitle == NULL) {
+		return NULL;
+	}
+
+	split = strstr (dtitle->str, " / ");
+	if (split == NULL) {
+		return CORBA_string_dup ("");
+	} else {
+		return CORBA_string_dup (split + 3);
+	}
+}
+
+static CORBA_short
+impl_GNOME_Media_CDDBSlave2_getNTrks (PortableServer_Servant servant,
+				      const CORBA_char *discid,
+				      CORBA_Environment *ev)
+{
+	CDDBEntry *entry;
+
+	entry = g_hash_table_lookup (cddb_cache, discid);
+	if (entry == NULL) {
+		return -1;
+	}
+
+	return entry->ntrks;
+}
+
+static CORBA_char *
+impl_GNOME_Media_CDDBSlave2_getTrackTitle (PortableServer_Servant servant,
+					   const CORBA_char *discid,
+					   CORBA_short track,
+					   CORBA_Environment *ev)
+{
+	CDDBEntry *entry;
+	char *name;
+	GString *ttitle;
+	
+	entry = g_hash_table_lookup (cddb_cache, discid);
+	if (entry == NULL) {
+		return NULL;
+	}
+
+	name = g_strdup_printf ("TTITLE%d", track);
+	ttitle = g_hash_table_lookup (entry->fields, name);
+	g_free (name);
+	if (ttitle == NULL) {
+		return NULL;
+	}
+
+	return CORBA_string_dup (ttitle->str);
+}
+
+static void
+impl_GNOME_Media_CDDBSlave2_getAllTracks (PortableServer_Servant servant,
+					  const CORBA_char *discid,
+					  GNOME_Media_CDDBSlave2_StringList **names_list,
+					  CORBA_Environment *ev)
+{
+	CDDBEntry *entry;
+	int ntrk;
+	
+	entry = g_hash_table_lookup (cddb_cache, discid);
+	if (entry == NULL) {
+		return;
+	}
+
+	*names_list = GNOME_Media_CDDBSlave2_StringList__alloc ();
+	(*names_list)->_length = 0;
+	(*names_list)->_maximum = entry->ntrks + 1;
+	(*names_list)->_buffer = CORBA_sequence_CORBA_string_allocbuf (entry->ntrks);
+
+	for (ntrk = 0; ntrk < entry->ntrks; ntrk++) {
+		char *name;
+		GString *ttitle;
+
+		name = g_strdup_printf ("TTITLE%d", ntrk);
+		ttitle = g_hash_table_lookup (entry->fields, name);
+		if (ttitle != NULL) {
+			(*names_list)->_buffer[ntrk] = CORBA_string_dup (ttitle->str ? ttitle->str : "");
+		} else {
+			(*names_list)->_buffer[ntrk] = CORBA_string_dup ("");
+		}
+		(*names_list)->_length++;
+
+		g_free (name);
+	}
+
+	return;
+}
+	
 
 static void
 finalize (GObject *object)
@@ -876,7 +1057,9 @@ finalize (GObject *object)
 		return;
 
 	g_free (priv->server);
-
+	g_free (priv->name);
+	g_free (priv->hostname);
+	
 	g_free (priv);
 	cddb->priv = NULL;
 
@@ -894,6 +1077,11 @@ cddb_slave_class_init (CDDBSlaveClass *klass)
 
 	parent_class = g_type_class_peek_parent (klass);
 	epv->query = impl_GNOME_Media_CDDBSlave2_query;
+	epv->getArtist = impl_GNOME_Media_CDDBSlave2_getArtist;
+	epv->getDiscTitle = impl_GNOME_Media_CDDBSlave2_getDiscTitle;
+	epv->getNTrks = impl_GNOME_Media_CDDBSlave2_getNTrks;
+	epv->getTrackTitle = impl_GNOME_Media_CDDBSlave2_getTrackTitle;
+	epv->getAllTracks = impl_GNOME_Media_CDDBSlave2_getAllTracks;
 }
 
 static void
@@ -903,6 +1091,10 @@ cddb_slave_init (CDDBSlave *cddb)
 
 	priv = g_new0 (CDDBSlavePrivate, 1);
 	cddb->priv = priv;
+
+	if (cddb_cache == NULL) {
+		cddb_cache = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+	}
 }
 
 BONOBO_TYPE_FUNC_FULL (CDDBSlave, GNOME_Media_CDDBSlave2, PARENT_TYPE, cddb_slave);
@@ -927,6 +1119,8 @@ BONOBO_TYPE_FUNC_FULL (CDDBSlave, GNOME_Media_CDDBSlave2, PARENT_TYPE, cddb_slav
 CDDBSlave *
 cddb_slave_new_full (const char *server,
 		     int port,
+		     const char *name,
+		     const char *hostname,
 		     BonoboEventSource *event_source)
 {
 	CDDBSlave *cddb;
@@ -949,6 +1143,9 @@ cddb_slave_new_full (const char *server,
 		priv->port = port;
 	}
 
+	priv->name = g_strdup (name);
+	priv->hostname = g_strdup (hostname);
+	
 	priv->event_source = event_source;
 	bonobo_object_add_interface (BONOBO_OBJECT (cddb),
 				     BONOBO_OBJECT (priv->event_source));
@@ -969,13 +1166,15 @@ cddb_slave_new_full (const char *server,
  */
 CDDBSlave *
 cddb_slave_new (const char *server,
-		int port)
+		int port,
+		const char *name,
+		const char *hostname)
 {
 	BonoboEventSource *event_source;
 
 	event_source = bonobo_event_source_new ();
 
-	return cddb_slave_new_full (server, port, event_source);
+	return cddb_slave_new_full (server, port, name, hostname, event_source);
 }
 
 /**
